/*
* Fabric3
* Copyright (c) 2009 Metaform Systems
*
* Fabric3 is free software: you can redistribute it and/or modify
* it under the terms of the GNU General Public License as
* published by the Free Software Foundation, either version 3 of
* the License, or (at your option) any later version, with the
* following exception:
*
* Linking this software statically or dynamically with other
* modules is making a combined work based on this software.
* Thus, the terms and conditions of the GNU General Public
* License cover the whole combination.
*
* As a special exception, the copyright holders of this software
* give you permission to link this software with independent
* modules to produce an executable, regardless of the license
* terms of these independent modules, and to copy and distribute
* the resulting executable under terms of your choice, provided
* that you also meet, for each linked independent module, the
* terms and conditions of the license of that module. An
* independent module is a module which is not derived from or
* based on this software. If you modify this software, you may
* extend this exception to your version of the software, but
* you are not obligated to do so. If you do not wish to do so,
* delete this exception statement from your version.
*
* Fabric3 is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty
* of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
* See the GNU General Public License for more details.
*
* You should have received a copy of the
* GNU General Public License along with Fabric3.
* If not, see <http://www.gnu.org/licenses/>.
*/
package org.fabric3.implementation.rs.runtime;

import java.lang.reflect.Method;
import java.util.Map;

import net.sf.cglib.proxy.MethodInterceptor;
import net.sf.cglib.proxy.MethodProxy;

import org.fabric3.spi.invocation.Message;
import org.fabric3.spi.invocation.MessageImpl;
import org.fabric3.spi.invocation.WorkContext;
import org.fabric3.spi.wire.Interceptor;
import org.fabric3.spi.wire.InvocationChain;

/**
 * Dispatches an invocation from Jersey to the Fabric3 invocation chain fronting a component instance.
 *
 * @version $Rev: 9270 $ $Date: 2010-08-01 14:07:13 +0000 (Sun, 01 Aug 2010) $
 */
public class RsMethodInterceptor implements MethodInterceptor {
    private Map<String, InvocationChain> invocationChains;

    public RsMethodInterceptor(Map<String, InvocationChain> invocationChains) {
        this.invocationChains = invocationChains;
    }

    public Object intercept(Object object, Method method, Object[] args, MethodProxy proxy) throws Throwable {
        Message message = new MessageImpl(args, false, new WorkContext());
        InvocationChain invocationChain = invocationChains.get(method.getName());
        if (invocationChain != null) {
            Interceptor headInterceptor = invocationChain.getHeadInterceptor();
            Message ret = headInterceptor.invoke(message);
            if (ret.isFault()) {
                throw (Throwable) ret.getBody();
            } else {
                return ret.getBody();
            }
        } else {
            return null;
        }
    }

}

